#[derive(PartialEq, Debug)]
pub struct Message {
    pub command: String,
    pub args: Vec<String>,
}

pub type Result<T> = std::result::Result<T, Error>;

#[repr(u8)]
#[derive(PartialEq, Debug)]
pub enum Error {
    BadParse,
}

impl Message {
    pub fn parse(text: String) -> Result<Message> {
	let re = regex::Regex::new(r"^([A-Z_]+):\s*(.*)").unwrap();
	match re.captures(text.as_str()) {
            Some(captures) => {
		if captures.len() < 3 {
		    Err(Error::BadParse)
		} else {
		    let command = captures.get(1).unwrap().as_str().to_string();
		    let args = captures.get(2).unwrap().as_str()
			.split(',')
			.map(|s| s.trim().to_string())
			.collect();
                    Ok(Message { command, args })
                }
            }
	    None => Err(Error::BadParse),
        }
    }
}

#[macro_export]
macro_rules! msg {
    ( $command:expr) => {
	{
	    let command = $command.to_string();
	    let args = Vec::new();
	    Message { command, args }
	}
    };
    ( $command:expr, $( $arg:expr ),*) => {
	{
	    let command = $command.to_string();
	    let args = vec![$($arg.to_string()),*];
	    Message { command, args }
	}
    };
}
	    

impl std::fmt::Display for Message {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::result::Result<(), std::fmt::Error> {
        if self.args.is_empty() {
            write!(f, "{}:", self.command)
        } else {
	    write!(f, "{}: {}", self.command, self.args.as_slice().join(", "))
        }
    }
}


#[cfg(test)]
mod test {
    use super::*;
    #[test]
    fn test_parse() -> Result<()> {
	let text = "COMMAND: arg1, arg2";
	let msg = Message::parse(text.to_string())?;
	assert_eq!(msg!("COMMAND", "arg1", "arg2"), msg);
	Ok(())
    }
    #[test]
    fn test_to_string() {
        let msg = msg!("COMMAND", "arg1", "arg2");
	assert_eq!(msg.to_string(), "COMMAND: arg1, arg2".to_string());
    }
}
